@@ -16,6 +16,9 @@ logger = logging.getLogger(__name__)
 COLL_LIBRARY_IDENTIFIER = 'library'
 # for backward compat, we should never change these filenames
 LIBRARY_FILENAME = f'{COLL_LIBRARY_IDENTIFIER}.fuo'
+DEPRECATED_FUO_FILENAMES = (
+    'Songs.fuo', 'Albums.fuo', 'Artists.fuo', 'Videos.fuo'
+)
 
 TOML_DELIMLF = "+++\n"
 
@@ -88,8 +91,9 @@ class Collection:
                 except ResolverNotFound:
                     logger.warning('resolver not found for line:%s', line)
                     model = None
-                except ResolveFailed:
-                    logger.warning('invalid line: %s', line)
+                except ResolveFailed as e:
+                    logger.warning('resolve failed, file:%s, line:%s, error:%s',
+                                   str(filepath), line, str(e))
                     model = None
                 if model is not None:
                     if model.exists is ModelExistence.no:
@@ -144,11 +148,10 @@ class Collection:
             return
         for i, model in enumerate(self.models.copy()):
             if model.exists is ModelExistence.no and model.source == provider.identifier:
-                model_cls = provider.get_model_cls(model.meta.model_type)
-                new_model = model_cls(model)
-                new_model.exists = ModelExistence.unknown
+                new_model = resolve(reverse(model, as_line=True))
                 # TODO: emit data changed signal
                 self.models[i] = new_model
+        # TODO: set _has_nonexistent_models to proper value
 
     def on_provider_removed(self, provider):
         for model in self.models:
@@ -177,14 +180,14 @@ class CollectionManager:
     def __init__(self, app):
         self._app = app
         self._library = app.library
+        self.default_dir = COLLECTIONS_DIR
 
     def scan(self):
-        """
-        scan collections directories for valid fuo files, yield
+        """Scan collections directories for valid fuo files, yield
         Collection instance for each file.
         """
         default_fpaths = []
-        directorys = [COLLECTIONS_DIR]
+        directorys = [self.default_dir]
         if self._app.config.COLLECTIONS_DIR:
             if isinstance(self._app.config.COLLECTIONS_DIR, list):
                 directorys += self._app.config.COLLECTIONS_DIR
@@ -199,7 +202,7 @@ class CollectionManager:
                 if not filename.endswith('.fuo'):
                     continue
                 filepath = os.path.join(directory, filename)
-                if filename in ('Songs.fuo', 'Albums.fuo', 'Artists.fuo', 'Videos.fuo'):
+                if filename in DEPRECATED_FUO_FILENAMES:
                     default_fpaths.append(filepath)
                     continue
                 coll = Collection(filepath)
@@ -216,13 +219,11 @@ class CollectionManager:
         yield coll
 
     def generate_library_coll_if_needed(self, default_fpaths):
-        library_fpath = os.path.join(COLLECTIONS_DIR, LIBRARY_FILENAME)
+        library_fpath = os.path.join(self.default_dir, LIBRARY_FILENAME)
         if os.path.exists(library_fpath):
             if default_fpaths:
                 paths_str = ','.join(default_fpaths)
-                logger.warning(f'{paths_str} and {library_fpath} '
-                               'should not exist at the same time')
-                logger.warning(f'{paths_str} are ignored')
+                logger.warning(f'{paths_str} are ignored since {library_fpath} exists')
             return library_fpath
 
         logger.info('Generating collection library')
@@ -231,7 +232,7 @@ class CollectionManager:
                  TOML_DELIMLF]
         if default_fpaths:
             for fpath in default_fpaths:
-                with open(fpath, 'w', encoding='utf-8') as f:
+                with open(fpath, 'r', encoding='utf-8') as f:
                     for line in f:
                         if line.startswith('fuo://'):
                             lines.append(line)
