@@ -9,21 +9,20 @@ import itertools
 import random
 import sys
 import logging
-from contextlib import suppress
-from requests.exceptions import RequestException
 
 try:
     # helper module should work in no-window mode
     from PyQt5.QtCore import QModelIndex, QSize, Qt, pyqtSignal, QSortFilterProxyModel, \
         QAbstractListModel
     from PyQt5.QtGui import QPalette, QFontMetrics
-    from PyQt5.QtWidgets import QApplication
+    from PyQt5.QtWidgets import QApplication, QScrollArea
 except ImportError:
     pass
 
 from feeluown.utils import aio
 from feeluown.excs import ProviderIOError
-from feeluown.library import NotSupported
+from feeluown.library import NotSupported, ModelType, BaseModel
+from feeluown.models.uri import reverse
 
 logger = logging.getLogger(__name__)
 
@@ -42,16 +41,6 @@ class ActionError(Exception):
     pass
 
 
-def use_mac_theme():
-    """判断是否需要使用 mac 主题
-
-    目前只是简单为 mac 做一些定制，但如果真的要引入 theme 这个概念，
-    单这一个函数是不够的。
-    """
-    return True
-    return sys.platform == 'darwin'
-
-
 def is_macos():
     """Check if operating system is macOS
 
@@ -96,6 +85,58 @@ class BgTransparentMixin:
         self.setPalette(palette)
 
 
+class BaseScrollAreaForNoScrollItemView(QScrollArea):
+    """A scroll area base class for itemview with no_scroll_v=True
+
+    Subclass must implement the following methods:
+    * itemview
+    * height_for_itemview
+    Also note the API is not stable.
+
+    .. versionadded:: 3.8.9
+    """
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        self.verticalScrollBar().valueChanged.connect(self.on_v_scrollbar_value_changed)
+
+    def get_itemview(self):
+        raise NotImplementedError
+
+    def height_for_itemview(self):
+        raise NotImplementedError
+
+    def resizeEvent(self, e):
+        super().resizeEvent(e)
+        self.maybe_resize_itemview()
+
+    def maybe_resize_itemview(self):
+        """Resize itemview to make sure it has a chance to fetch more items
+
+        When the itemview has no more items, do not need to resize it.
+        """
+        itemview = self.get_itemview()
+        if itemview is not None:
+            model = itemview.model()
+            if model is not None and model.canFetchMore(QModelIndex()):
+                # +1 to make sure user can scroll, then user has a change to
+                # trigger itemview fetch more.
+                itemview.suggest_min_height(self.height_for_itemview() + 1)
+                itemview.adjust_height()
+
+    def maybe_trigger_itemview_fetch_more(self):
+        itemview = self.get_itemview()
+        if itemview is not None:
+            model = itemview.model()
+            if model is not None and model.canFetchMore(QModelIndex()):
+                model.fetchMore(QModelIndex())
+
+    def on_v_scrollbar_value_changed(self, value):
+        maximum = self.verticalScrollBar().maximum()
+        if maximum == value:
+            self.maybe_trigger_itemview_fetch_more()
+
+
 class ItemViewNoScrollMixin:
     """
     `no_scroll_v` means that the itemview's size(height) is always enough to hold
@@ -105,14 +146,19 @@ class ItemViewNoScrollMixin:
     The itemview with no_scroll_v=True is usually used with an outside ScrollArea.
     """
     def __init__(self, *args, no_scroll_v=True, row_height=0, least_row_count=0,
-                 reserved=30, **kwargs):
+                 fixed_row_count=0, reserved=30, **kwargs):
         """
         :params no_scroll_v: enable on no_scroll_v feature or not
+        :params fixed_row_count: set row_height when fixed_row_count is set
 
         .. versionadded:: 3.7.8
            The *row_height*, *least_row_count*, *reserved* parameter were added.
+
+        .. versionadded:: 3.8.9
+           The *fixed_row_count* parameter was added.
         """
         self._least_row_count = least_row_count
+        self._fixed_row_count = fixed_row_count
         self._row_height = row_height
         self._reserved = reserved
 
@@ -141,15 +187,19 @@ class ItemViewNoScrollMixin:
             return
 
         if self.model().canFetchMore(QModelIndex()):
-            # according to QAbstractItemView source code,
-            # qt will trigger fetchMore when the last row is
-            # inside the viewport, so we always hide the last
-            # two row to ensure fetch-more will not be
-            # triggered automatically
-            index = self._last_visible_index()
-            rect = self.visualRect(index)
-            height = self.sizeHint().height() - int(rect.height() * 1.5) - self._reserved
-            self.setFixedHeight(max(height, self.min_height()))
+            if self._fixed_row_count == 0:
+                # according to QAbstractItemView source code,
+                # qt will trigger fetchMore when the last row is
+                # inside the viewport, so we always hide the last
+                # two row to ensure fetch-more will not be
+                # triggered automatically
+                index = self._last_visible_index()
+                rect = self.visualRect(index)
+                height = self.sizeHint().height() - int(rect.height() * 1.5) - \
+                    self._reserved
+                self.setFixedHeight(max(height, self.min_height()))
+            else:
+                self.setFixedHeight(self._row_height * self._fixed_row_count)
         else:
             height = self.sizeHint().height()
             self.setFixedHeight(height)
@@ -187,10 +237,13 @@ class ItemViewNoScrollMixin:
 
         height = min_height = self.min_height()
         if self.model() is not None:
-            index = self._last_visible_index()
-            rect = self.visualRect(index)
-            height = rect.y() + rect.height() + self._reserved
-            height = max(min_height, height)
+            if self._fixed_row_count == 0:
+                index = self._last_visible_index()
+                rect = self.visualRect(index)
+                height = rect.y() + rect.height() + self._reserved
+                height = max(min_height, height)
+            else:
+                height = self._row_height * self._fixed_row_count
         return QSize(super_size_hint.width(), height)
 
     def _last_visible_index(self):
@@ -341,22 +394,70 @@ class ReaderFetchMoreMixin:
 
 def fetch_cover_wrapper(app):
     """
-    Your should only use this helper within ImgListModel.
+    Your should only use this helper within ImgListModel and SongMiniCardListModel.
     """
     img_mgr, library = app.img_mgr, app.library
 
-    async def fetch_model_cover(model, cb, uid):
-        # try get from cache first
-        content = img_mgr.get_from_cache(uid)
+    async def fetch_model_cover(model, cb):
+        # Get image unique id.
+        model_is_song = False
+        song_is_v2 = False
+        upgraded_song = None
+        if ModelType(model.meta.model_type) is ModelType.song:
+            model_is_song = True
+            if isinstance(model, BaseModel):
+                song_is_v2 = True
+                img_uid, _ = model.cache_get('album_uid')
+            else:
+                img_uid = None
+        else:
+            img_uid = reverse(model) + '/cover'
+        if img_uid is None:
+            assert model_is_song
+            try:
+                upgraded_song = await aio.run_fn(library.song_upgrade, model)
+                album = upgraded_song.album
+            except NotSupported:
+                album = None
+            if album is None:
+                cb(None)
+                return
+
+            img_uid = reverse(album) + '/cover'
+            if song_is_v2:
+                model.cache_set('album_uid', img_uid)
+
+        # Check image cache with image unique ID.
+        content = img_mgr.get_from_cache(img_uid)
         if content is not None:
-            return cb(content)
-        # FIXME: sleep random second to avoid send too many request to provider
-        await asyncio.sleep(random.randrange(100) / 100)
-        with suppress(ProviderIOError, RequestException, NotSupported):
-            url = await aio.run_fn(library.model_get_cover, model)
-            if url:
-                content = await img_mgr.get(url, uid)
-                cb(content)
+            cb(content)
+            return
+
+        # Get image url.
+        img_url = None
+        if model_is_song and song_is_v2:
+            img_url, _ = model.cache_get('album_cover')
+        if img_url is None:
+            if model_is_song:
+                model_with_img = upgraded_song.album
+            else:
+                model_with_img = model
+            try:
+                img_url = await aio.run_fn(library.model_get_cover, model_with_img)
+            except NotSupported:
+                img_url = ''
+            if model_is_song:
+                model.cache_set('album_cover', img_url)
+
+        # Fetch image by url and invoke cb.
+        if img_url:
+            # FIXME: sleep random second to avoid send too many request to provider
+            await asyncio.sleep(random.randrange(100) / 100)
+            content = await img_mgr.get(img_url, img_uid)
+            cb(content)
+        else:
+            cb(None)
+
     return fetch_model_cover
 
 
