@@ -4,7 +4,7 @@ import warnings
 from contextlib import suppress
 
 from PyQt5.QtCore import Qt
-from PyQt5.QtGui import QImage, QPixmap
+from PyQt5.QtGui import QImage, QPixmap, QPalette
 from PyQt5.QtWidgets import QFrame, QVBoxLayout, QLabel, QApplication
 from requests.exceptions import RequestException
 
@@ -239,6 +239,8 @@ class DescLabel(QLabel):
         self.setWordWrap(True)
         self.setTextInteractionFlags(Qt.TextSelectableByMouse)
 
+        self.palette().setColor(QPalette.Background, self.palette().color(QPalette.Base))
+
 
 class TableContainer(QFrame, BgTransparentMixin):
     def __init__(self, app, parent=None):
@@ -253,7 +255,7 @@ class TableContainer(QFrame, BgTransparentMixin):
         self.toolbar = SongsTableToolbar()
         self.tabbar = TableTabBarV2()
         self.meta_widget = TableMetaWidget(parent=self)
-        self.songs_table = SongsTableView(parent=self)
+        self.songs_table = SongsTableView(app=self._app, parent=self)
         self.albums_table = AlbumListView(parent=self)
         self.artists_table = ArtistListView(parent=self)
         self.videos_table = VideoListView(parent=self)
@@ -304,7 +306,6 @@ class TableContainer(QFrame, BgTransparentMixin):
 
         self._v_layout.addWidget(self.meta_widget)
         self._v_layout.addWidget(self.toolbar)
-        self._v_layout.addSpacing(10)
         self._v_layout.addWidget(self.desc_widget)
 
         # Since QTableView has a margin on the left and right(see SongsTableDelegate),
@@ -494,9 +495,9 @@ class TableContainer(QFrame, BgTransparentMixin):
             try:
                 song = await aio.run_in_executor(
                     None, self._app.library.song_upgrade, song)
-            except NotSupported:
+            except NotSupported as e:
                 assert ModelFlags.v2 & song.meta.flags
-                self._app.show_msg('资源提供放不支持该功能')
+                self._app.show_msg(f'资源提供方不支持该功能: {str(e)}')
                 logger.info(f'provider:{song.source} does not support song_get')
                 song.state = ModelState.cant_upgrade
             except (ProviderIOError, RequestException) as e:
