@@ -1,7 +1,9 @@
 import logging
+import warnings
 from functools import partial, lru_cache
-from typing import List, cast, Optional, Union
+from typing import cast, Optional, Union
 
+from feeluown.excs import ModelCannotUpgrade
 from feeluown.media import Media
 from feeluown.models import SearchType, ModelType
 from feeluown.utils import aio
@@ -17,12 +19,21 @@ from .excs import (
 from .flags import Flags as PF
 from .models import (
     ModelFlags as MF, BaseModel, BriefSongModel, UserModel,
-    get_modelcls_by_type, V2SupportedModelTypes,
+    get_modelcls_by_type,
 )
 from .model_protocol import (
     BriefVideoProtocol, ModelProtocol, BriefSongProtocol, SongProtocol, UserProtocol,
     LyricProtocol, VideoProtocol, BriefAlbumProtocol, BriefArtistProtocol
 )
+from .model_state import ModelState
+from .provider_protocol import (
+    check_flag as check_flag_impl,
+    SupportsCurrentUser,
+    SupportsSongLyric, SupportsSongMV, SupportsSongMultiQuality,
+    SupportsPlaylistRemoveSong, SupportsPlaylistAddSong, SupportsPlaylistSongsReader,
+    SupportsArtistSongsReader, SupportsArtistAlbumsReader,
+    SupportsVideoMultiQuality, SupportsArtistContributedAlbumsReader,
+)
 
 
 logger = logging.getLogger(__name__)
@@ -31,6 +42,11 @@ FULL_SCORE = 10
 MIN_SCORE = 5
 
 
+def support_or_raise(provider, protocol_cls):
+    if not isinstance(provider, protocol_cls):
+        raise NotSupported(f'{provider} not support {protocol_cls}') from None
+
+
 def default_score_fn(origin, standby):
 
     # TODO: move this function to utils module
@@ -161,7 +177,7 @@ class Library:
         else:
             self.provider_removed.emit(provider)
 
-    def get(self, identifier) -> Optional[Union[AbstractProvider, ProviderV2]]:
+    def get(self, identifier):
         """通过资源提供方唯一标识获取提供方实例"""
         for provider in self._providers:
             if provider.identifier == identifier:
@@ -366,7 +382,7 @@ class Library:
             raise ProviderNotFound(f'provider {identifier} not found')
         return provider
 
-    def getv2_or_raise(self, identifier) -> ProviderV2:
+    def getv2_or_raise(self, identifier):
         provider = self.get_or_raise(identifier)
         # You should ensure the provider is v2 first. For example, if check_flags
         # returns true, the provider must be a v2 instance.
@@ -386,13 +402,17 @@ class Library:
         provider = self.get(source)
         if provider is None:
             return False
-        if isinstance(provider, ProviderV2):
-            return provider.check_flags(model_type, flags)
-        # Always return false when the provider is not a v2 instance.
-        return False
+
+        # Check each flag.
+        for value in PF.__members__.values():
+            if value in PF(flags):
+                if check_flag_impl(provider, model_type, flags) is False:
+                    return False
+        return True
 
     def check_flags_by_model(self, model: ModelProtocol, flags: PF) -> bool:
         """Alias for check_flags"""
+        warnings.warn('please use isintance(provider, protocol_cls)')
         return self.check_flags(model.source,
                                 ModelType(model.meta.model_type),
                                 flags)
@@ -432,23 +452,14 @@ class Library:
     def song_upgrade(self, song: BriefSongProtocol) -> SongProtocol:
         return self._model_upgrade(song)  # type: ignore
 
-    def song_list_similar(self, song: BriefSongProtocol) -> List[BriefSongProtocol]:
-        provider = self.getv2_or_raise(song.source)
-        return provider.song_list_similar(song)
-
-    def song_list_hot_comments(self, song: BriefSongProtocol):
-        provider = self.getv2_or_raise(song.source)
-        return provider.song_list_hot_comments(song)
-
     def song_prepare_media(self, song: BriefSongProtocol, policy) -> Media:
         provider = self.get(song.source)
         if provider is None:
             # FIXME: raise ProviderNotfound
             raise MediaNotFound(f'provider:{song.source} not found')
         if song.meta.flags & MF.v2:
-            # provider MUST has multi_quality flag for song
-            assert self.check_flags_by_model(song, PF.multi_quality)
-            assert isinstance(provider, ProviderV2)
+            support_or_raise(provider, SupportsSongMultiQuality)
+            provider = cast(SupportsSongMultiQuality, provider)
             media, _ = provider.song_select_media(song, policy)
         else:
             if song.meta.support_multi_quality:
@@ -468,19 +479,11 @@ class Library:
 
         .. versionadded:: 3.7.5
         """
-        song_v1 = self.cast_model_to_v1(song)
-        mv = song_v1.mv
-        if mv.meta.support_multi_quality:
-            media, _ = mv.select_media(policy)
-        else:
-            media = mv.media
-            if media:
-                media = Media(media)
-            else:
-                media = None
-        if not media:
-            raise MediaNotFound
-        return media
+        mv = self.song_get_mv(song)
+        if mv is not None:
+            media = self.video_prepare_media(mv, policy)
+            return media
+        raise MediaNotFound
 
     def song_get_mv(self, song: BriefSongProtocol) -> Optional[VideoProtocol]:
         """
@@ -489,13 +492,14 @@ class Library:
         :raises ProviderNotFound:
         """
         provider = self.get_or_raise(song.source)
-        if self.check_flags_by_model(song, PF.mv):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsSongMV):
             mv = provider.song_get_mv(song)
-        else:
+        elif not self.check_flags(song.source, song.meta.model_type, PF.model_v2):
             song_v1 = self.cast_model_to_v1(song)
             mv = song_v1.mv
             mv = cast(Optional[VideoProtocol], mv)
+        else:
+            mv = None
         return mv
 
     def song_get_lyric(self, song: BriefSongModel) -> Optional[LyricProtocol]:
@@ -508,13 +512,12 @@ class Library:
         :raises ProviderNotFound:
         """
         provider = self.get_or_raise(song.source)
-        if self.check_flags_by_model(song, PF.lyric):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsSongLyric):
             lyric = provider.song_get_lyric(song)
         else:
             song_v1 = self.cast_model_to_v1(song)
-            lyric_v1 = song_v1.lyric
-            lyric = cast(Optional[LyricProtocol], lyric_v1)
+            lyric = song_v1.lyric
+            lyric = cast(Optional[LyricProtocol], lyric)
         return lyric
 
     def song_get_web_url(self, song: BriefSongProtocol) -> str:
@@ -539,8 +542,7 @@ class Library:
         :raises NotSupported:
         """
         provider = self.get_or_raise(artist.source)
-        if self.check_flags_by_model(artist, PF.songs_rd):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsArtistSongsReader):
             reader = provider.artist_create_songs_rd(artist)
         else:
             artist = self.cast_model_to_v1(artist)
@@ -550,19 +552,30 @@ class Library:
                 reader = create_reader(artist.songs)
         return reader
 
-    def artist_create_albums_rd(self, artist):
+    def artist_create_albums_rd(self, artist, contributed=False):
         """Create albums reader for artist model.
         """
         provider = self.get_or_raise(artist.source)
-        if self.check_flags_by_model(artist, PF.albums_rd):
-            assert isinstance(provider, ProviderV2)
-            reader = provider.artist_create_albums_rd(artist)
-        else:
-            artist = self.cast_model_to_v1(artist)
-            if artist.meta.allow_create_albums_g:
-                reader = create_reader(artist.create_albums_g())
+        if contributed is False:
+            if isinstance(provider, SupportsArtistAlbumsReader):
+                reader = provider.artist_create_albums_rd(artist)
             else:
-                raise NotSupported("can't create albums reader for artist")
+                artist = self.cast_model_to_v1(artist)
+                if artist.meta.allow_create_albums_g:
+                    reader = create_reader(artist.create_albums_g())
+                else:
+                    raise NotSupported("can't create albums reader for artist")
+        else:
+            if isinstance(provider, SupportsArtistContributedAlbumsReader):
+                reader = provider.artist_create_contributed_albums_rd(artist)
+            else:
+                artist = self.cast_model_to_v1(artist)
+                # Old code check if provider supports contributed_albums in this way.
+                if hasattr(artist, 'contributed_albums') and artist.contributed_albums:
+                    reader = create_reader(artist.create_contributed_albums_g())
+                else:
+                    raise NotSupported(
+                        "can't create contributed albums reader for artist")
         return reader
 
     # --------
@@ -578,8 +591,7 @@ class Library:
         :raises NotSupported:
         """
         provider = self.get_or_raise(playlist.source)
-        if self.check_flags_by_model(playlist, PF.songs_rd):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsPlaylistSongsReader):
             reader = provider.playlist_create_songs_rd(playlist)
         else:
             playlist = self.cast_model_to_v1(playlist)
@@ -595,8 +607,7 @@ class Library:
         :return: true if the song is not in playlist anymore.
         """
         provider = self.get_or_raise(playlist.source)
-        if self.check_flags_by_model(playlist, PF.remove_song):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsPlaylistRemoveSong):
             ok = provider.playlist_remove_song(playlist, song)
         else:
             playlist = self.cast_model_to_v1(playlist)
@@ -609,8 +620,7 @@ class Library:
         :return: true if the song exists in playlist.
         """
         provider = self.get_or_raise(playlist.source)
-        if self.check_flags_by_model(playlist, PF.remove_song):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsPlaylistAddSong):
             ok = provider.playlist_add_song(playlist, song)
         else:
             playlist = self.cast_model_to_v1(playlist)
@@ -635,8 +645,8 @@ class Library:
         model = None
         try_v1way = True
         if isinstance(provider, ProviderV2):
-            if provider.check_flags(mtype, PF.model_v2):
-                if provider.check_flags(mtype, PF.get):
+            if provider.use_model_v2(mtype):
+                if self.check_flags(pid, mtype, PF.get):
                     try_v1way = False
                     model = provider.model_get(mtype, mid)
 
@@ -657,20 +667,19 @@ class Library:
         :param model: model which has a 'cover' field.
         :return: cover url if exists, else ''.
         """
-        cover = ''
         if MF.v2 in model.meta.flags:
-            if MF.normal in model.meta.flags:
-                cover = model.cover
-            else:
-                # TODO: upgrade artist model.
-                # Currently supported model types: (ModelType.album, ModelType.video).
-                if ModelType(model.meta.model_type) in V2SupportedModelTypes:
+            if MF.normal not in model.meta.flags:
+                try:
                     um = self._model_upgrade(model)
-                    # FIXME: remove this hack lator.
-                    if ModelType(model.meta.model_type) is ModelType.artist:
-                        cover = um.pic_url
-                    else:
-                        cover = um.cover
+                except (ModelCannotUpgrade, NotSupported):
+                    return ''
+            else:
+                um = model
+            # FIXME: remove this hack lator.
+            if ModelType(model.meta.model_type) is ModelType.artist:
+                cover = um.pic_url
+            else:
+                cover = um.cover
         else:
             cover = model.cover
             # Check if cover is a media object.
@@ -680,37 +689,42 @@ class Library:
 
     def _model_upgrade(self, model):
         """
-        Thinking: currently, the caller must catch the NotSupported error.
+        :raises NotSupported: provider does't impl SupportGetProtocol for the model type
+        :raises CannotUpgrade: the model can't be upgraded
         """
-        try_v1way = True
-        upgraded_model = None
-        if model.meta.flags & MF.v2:
-            if MF.normal in model.meta.flags:
-                upgraded_model = model
-                try_v1way = False
-            else:
-                provider = self.getv2_or_raise(model.source)
-                # When the provider does not support 'get' for this model.
-                # Do not raise NotSupported here and try to use the v1 way.
-                #
-                # For example, provider X may support 'get' for SongModel, then
-                # the song.artists can return list of BriefArtistModel.
-                if self.check_flags_by_model(model, PF.get):
-                    upgraded_model = provider.model_get(
-                        model.meta.model_type, model.identifier)
-                    try_v1way = False
+        # Upgrade model in v1 way if it is a v1 model.
+        if MF.v2 not in model.meta.flags:
+            return self._model_upgrade_in_v1_way(model)
 
-        if try_v1way is True:
-            v1model = self.cast_model_to_v1(model)
-            modelcls = get_modelcls_by_type(ModelType(model.meta.model_type))
-            fields = [f for f in list(modelcls.__fields__)
-                      if f not in list(BaseModel.__fields__)]
-            for field in fields:
-                getattr(v1model, field)
-            upgraded_model = v1model
-        else:
-            assert upgraded_model is not None
-        return upgraded_model
+        # Return model directly if it is already a normal model.
+        if MF.normal in model.meta.flags:
+            return model
+
+        provider = self.getv2_or_raise(model.source)
+        model_type = ModelType(model.meta.model_type)
+        is_support = check_flag_impl(provider, model_type, PF.get)
+        if is_support:
+            upgraded_model = provider.model_get(model_type, model.identifier)
+            if upgraded_model is None:
+                model.state = ModelState.not_exists
+                raise ModelCannotUpgrade('model does not exist')
+            return upgraded_model
+
+        # Fallback to v1 way if the provider does not support PF.get.
+        # For example, provider X may support 'get' for SongModel and it
+        # does not support 'get' for ArtistModel temporarily. It returns
+        # a SongModel and the song.artists returns list of BriefArtistModel,
+        # in this condition, BriefArtistModel should be upgraded in v1 way.
+        return self._model_upgrade_in_v1_way(model)
+
+    def _model_upgrade_in_v1_way(self, model):
+        v1model = self.cast_model_to_v1(model)
+        modelcls = get_modelcls_by_type(ModelType(model.meta.model_type))
+        fields = [f for f in list(modelcls.__fields__)
+                  if f not in list(BaseModel.__fields__)]
+        for field in fields:
+            getattr(v1model, field)
+        return v1model
 
     # --------
     # Video
@@ -719,11 +733,14 @@ class Library:
         return self._model_upgrade(video)
 
     def video_prepare_media(self, video: BriefVideoProtocol, policy) -> Media:
+        """Prepare media for video.
+
+        :param video: either a v1 MvModel or a v2 (Brief)VideoModel.
+        """
         provider = self.get_or_raise(video.source)
         if video.meta.flags & MF.v2:
             # provider MUST has multi_quality flag for video
-            assert self.check_flags_by_model(video, PF.multi_quality)
-            assert isinstance(provider, ProviderV2)
+            assert isinstance(provider, SupportsVideoMultiQuality)
             media, _ = provider.video_select_media(video, policy)
         else:
             # V1 VideoModel has attribute `media`
@@ -746,8 +763,7 @@ class Library:
         .. versionadded:: 3.7.6
         """
         provider = self.get_or_raise(source)
-        if self.check_flags(source, ModelType.none, PF.current_user):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsCurrentUser):
             return provider.has_current_user()
 
         try:
@@ -768,8 +784,7 @@ class Library:
         .. versionadded:: 3.7.6
         """
         provider = self.get_or_raise(source)
-        if self.check_flags(source, ModelType.none, PF.current_user):
-            assert isinstance(provider, ProviderV2)
+        if isinstance(provider, SupportsCurrentUser):
             return provider.get_current_user()
 
         user_v1 = getattr(provider, '_user', None)
