@@ -264,6 +264,11 @@ class SongModel(BaseNormalModel):
     artists: List[BriefArtistModel]
     duration: int  # milliseconds
 
+    genre: str = ''
+    date: str = ''  # For example: 2020-12-11 00:00:00, 2020-12-11T00:00:00Z
+    track: str = '1/1'  # The number of the track on the album.
+    disc: str = '1/1'
+
     @property
     def artists_name(self):
         return fmt_artists(self.artists)
@@ -369,6 +374,15 @@ class PlaylistModel(BaseBriefModel):
     description: str
 
 
+class SimpleSearchResult(_BaseModel):
+    q: str
+    songs: List[BriefSongModel] = []
+    albums: List[BriefAlbumModel] = []
+    artists: List[BriefAlbumModel] = []
+    playlists: List[BriefPlaylistModel] = []
+    videos: List[BriefVideoModel] = []
+
+
 _type_modelcls_mapping = {
     ModelType.song: (SongModel, BriefSongModel),
     ModelType.album: (AlbumModel, BriefAlbumModel),
