@@ -6,23 +6,6 @@ from .models import V2SupportedModelTypes
 from .flags import Flags
 from .excs import MediaNotFound, ModelNotFound, NoUserLoggedIn, \
     NotSupported  # noqa
-from .provider_protocol import check_flag as check_flag_by_protocol
-
-
-def check_flags(provider, model_type: ModelType, flags: Flags):
-    """
-
-    Example::
-
-        {
-           ModelType.song: [Flags.get],
-           None: [Flags.current_user]  # flags that is not related to any model
-        }
-    """
-    # TODO: also check other types with protocol.
-    if model_type is ModelType.song and flags is not Flags.model_v2:
-        return check_flag_by_protocol(provider, model_type, flags)
-    return provider.meta.flags.get(model_type, Flags.none) & flags
 
 
 class ProviderV2:
@@ -31,9 +14,42 @@ class ProviderV2:
     class meta:
         identifier: str = ''
         name: str = ''
+
+        #: deprecated: use provider protocol instead.
         flags: dict = {}
 
-    check_flags = check_flags
+    def use_model_v2(self, model_type: ModelType) -> bool:
+        """Check whether model v2 is used for the specified model_type.
+
+        For feeluown developer, there are three things you should know.
+
+        1. Both v2 model and v1 model implement BriefXProtocol, which means
+           model.{attirbute}_display works for both models. For exmample,
+           SongModel(v2), BriefSongModel(v2) and SongModel(v1) all implement
+           BriefSongProtocol. So no matter which version the `song` is, it is
+           always safe to access `song.title_display`.
+
+        2. When model v2 is used, it means the way of accessing model's attributes
+           becomes different. So you should always check which version
+           the model is before accessing some attributes.
+
+           For model v1, you can access all model's attributes by {model}.{attribute},
+           and IO(network) operations may be performed implicitly. For example,
+           the code `song.url` *may* trigger a network request to fetch the
+           url when `song.url` is currently None. Tips: you can check the
+           `BaseModel.__getattribute__` implementation in `feeluown.models` package
+           for more details.
+
+           For model v2, everything are explicit. Basic attributes of model can be
+           accessed by {model}.{attribute} and there will be no IO operations.
+           Other attributes can only be accessed with methods of library. For example,
+           you can access song url/media info by `library.song_prepare_media`.
+
+        3. When deserializing model from a text line, the model version is important.
+           If provider does not declare it uses model v2, feeluown just use model v1
+           to do deserialization to keep backward compatibility.
+        """
+        return Flags.model_v2 in self.meta.flags.get(model_type, Flags.none)
 
     def model_get(self, model_type, model_id):
         if model_type in V2SupportedModelTypes:
