@@ -11,6 +11,7 @@ from feeluown.utils.dispatch import Signal
 from feeluown.utils.utils import DedupList
 from feeluown.player import Metadata, MetadataFields
 from feeluown.library import MediaNotFound, SongProtocol, ModelType, NotSupported
+from feeluown.media import Media
 from feeluown.models.uri import reverse
 
 logger = logging.getLogger(__name__)
@@ -440,8 +441,9 @@ class Playlist:
         except ProviderIOError as e:
             # FIXME: This may cause infinite loop when the prepare media always fails
             logger.error(f'prepare media failed: {e}, try next song')
-        except:  # noqa
+        except Exception as e:  # noqa
             # When the exception is unknown, we mark the song as bad.
+            self._app.show_msg(f'prepare media failed due to unknown error: {e}')
             logger.exception('prepare media failed due to unknown error, '
                              'so we mark the song as a bad one')
             self.mark_as_bad(song)
@@ -491,13 +493,20 @@ class Playlist:
         try:
             song = await aio.run_fn(self._app.library.song_upgrade, song)
             if song.album is not None:
-                album = self._app.library.album_upgrade(song.album)
+                album = await aio.run_fn(self._app.library.album_upgrade, song.album)
                 artwork = album.cover
+                # For model v1, the cover can be a Media object.
+                # For example, in fuo_local plugin, the album.cover is a Media
+                # object with url set to fuo://local/songs/{identifier}/data/cover.
+                if isinstance(artwork, Media):
+                    artwork = artwork.url
             else:
                 artwork = ''
         except NotSupported:
             # The song or the album can't be upgraded.
             pass
+        except:  # noqa
+            logger.exception(f"prepare metadata for song '{str(song)}' failed")
         else:
             if artwork:
                 metadata[MetadataFields.artwork] = artwork
